/// <reference types="node" />
import { AxiosRequestConfig, AxiosRequestHeaders } from 'axios';
import { ReadStream } from 'fs';

declare class BunnyCdnStreamVideo {
    videoLibraryId: number;
    guid: string;
    title: string;
    dateUploaded: string;
    views: number;
    isPublic: boolean;
    length: number;
    status: number;
    framerate: number;
    width: number;
    height: number;
    availableResolutions: string;
    thumbnailCount: number;
    encodeProgress: number;
    storageSize: number;
    captions: {
        srclang: string;
        label: string;
    }[];
    hasMP4Fallback: boolean;
    collectionId: '';
    thumbnailFileName: string;
    averageWatchTime: number;
    totalWatchTime: number;
    category: string;
    chapters: {
        title: string;
        start: number;
        end: number;
    }[];
    moments: {
        label: string;
        timestamp: number;
    }[];
    metaTags: {
        property: string;
        value: string;
    }[];
    constructor(data: BunnyCdnStream.VideoResponse);
    get resolutions(): number[];
}

declare class BunnyCdnStream {
    axiosOptions: BunnyCdnStream.BunnyAxiosRequestConfig;
    /**
     * Options for connecting and authenticating with the Bunny CDN API
     */
    options: BunnyCdnStream.Options;
    constructor(options: BunnyCdnStream.Options);
    /**
     * Retrieve a video from BunnyCdn
     * @returns A {@link BunnyCdnStream.VideoResponse} instance.
     * @param videoId The video ID
     * @example
     * ```typescript
     * await stream.getVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    getVideo(videoId: string): Promise<BunnyCdnStreamVideo>;
    /**
     * Update video information
     * @returns A {@link BunnyCdnStream.VideoResponse} instance.
     * @param videoId The video ID
     * @param data The data to update
     * @example
     * ```typescript
     * await stream.updateVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { title: "New title" })
     * ```
     */
    updateVideo(videoId: string, data?: {
        title?: string;
        collectionId?: string;
        chapters?: {
            title: string;
            start: number;
            end: number;
        }[];
        moments?: {
            label: string;
            timestamp: number;
        }[];
        metaTags?: {
            property: string;
            value: string;
        }[];
    }): Promise<BunnyCdnStream.UpdateVideoResponse>;
    /**
     * Delete a video
     * @returns A {@link BunnyCdnStream.DeleteVideoResponse} instance.
     * @param videoId The video ID
     * @example
     * ```typescript
     * await stream.deleteVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    deleteVideo(videoId: string): Promise<BunnyCdnStream.DeleteVideoResponse>;
    /**
     * Delete all videos
     *
     * NOTE: This uses the listVideos method and will iterate over all pages and delete all videos per page before continuing to the next page.
     * @returns void
     * @example
     * ```typescript
     * await stream.deleteAllVideos();
     * ```
     */
    deleteAllVideos(): Promise<void>;
    /**
     * Create a video, this does not upload the video file
     * @returns A {@link BunnyCdnStream.VideoResponse} instance.
     * @param data The data to create the video with
     * @example
     * ```typescript
     * await stream.createVideo({ title: "The best title" })
     * ```
     */
    createVideo(data: {
        title: string;
        collectionId?: string;
    }): Promise<BunnyCdnStreamVideo>;
    /**
     * Upload video, this does not create the video and requires a created video
     * @returns A {@link BunnyCdnStream.UploadVideoResponse} instance.
     * @param file The video file to upload as a readable stream
     * @param videoId The video id to upload to of a created video
     * @param data Optional paramaters such as enabledResolutions
     * @example
     * ```typescript
     * await stream.uploadVideo(createReadStream("./file.mp4"), "0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    uploadVideo(file: ReadStream, videoId: string, data?: {
        enabledResolutions?: string;
    }): Promise<BunnyCdnStream.UploadVideoResponse>;
    /**
     * Create and upload a video in one function
     * @returns A {@link BunnyCdnStream.VideoResponse} instance.
     * @param file The video file to upload as a readable stream
     * @param data The data to create the video with
     * @example
     * ```typescript
     * await stream.createAndUploadVideo(createReadStream("./file.mp4"), { title: "The best title" })
     * ```
     */
    createAndUploadVideo(file: ReadStream, data: {
        title: string;
        collectionId?: string;
    }): Promise<BunnyCdnStreamVideo>;
    /**
     * Get video statistics
     * @returns A {@link BunnyCdnStream.VideoHeatmapResponse} instance.
     * @param videoId The video id to get heatmap info from
     * @example
     * ```typescript
     * await stream.getVideoHeatmap("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    getVideoHeatmap(videoId: string): Promise<BunnyCdnStream.VideoHeatmapResponse>;
    /**
     * Get video statistics
     * @returns A {@link BunnyCdnStream.VideoStatisticsResponse} instance.
     * @param data The data to fetch video statistics with
     * @example
     * ```typescript
     * await stream.getVideoStatistics("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    getVideoStatistics(videoId: string, data?: {
        hourly?: boolean;
        dateTo?: string;
        dateFrom?: string;
    }): Promise<BunnyCdnStream.VideoStatisticsResponse>;
    /**
     * Force re-encode a video
     *
     * NOTE: This will not work if keepOriginal is set to false
     * @returns A {@link BunnyCdnStream.VideoResponse} instance.
     * @param videoId The video ID
     * @example
     * ```typescript
     * await stream.reencodeVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    reencodeVideo(videoId: string): Promise<BunnyCdnStreamVideo>;
    /**
     * List videos
     * @returns An array of {@link BunnyCdnStream.VideoStatisticsResponse} instances.
     * @param data The options to list videos with
     * @example
     * ```typescript
     * await stream.listVideos({ page: 2, search: "The best title", itemsPerPage: 10 })
     * ```
     */
    listVideos(data?: {
        page?: number;
        itemsPerPage?: number;
        search?: string;
        collection?: string;
        orderBy?: string;
    }): Promise<{
        items: BunnyCdnStreamVideo[];
        totalItems: number;
        currentPage: number;
        itemsPerPage: number;
    }>;
    /**
     * List all videos with an optional callback between each page
     * @returns An array of {@link BunnyCdnStream.VideoStatisticsResponse} instances.
     * @param data The options to list videos with
     * @param stop The callback that if returns ``true`` stops the iteration
     * @example
     * ```typescript
     * await stream.listAllVideos()
     * ```
     */
    listAllVideos(data?: {
        search?: string;
        collection?: string;
        orderBy?: string;
        itemsPerPage?: number;
    }, stop?: (videos: BunnyCdnStreamVideo[], page: number, totalPages: number) => boolean): Promise<BunnyCdnStreamVideo[]>;
    /**
     * Set the thumbnail
     *
     * NOTE: The file type is automatically detected from the buffer, however if it fails, it will default to ``image/jpeg``
     * @returns A {@link BunnyCdnStream.SetThumbnailVideoResponse} instance.
     * @param videoId The video ID
     * @param thumbnail A buffer/stream/url of the thumbnail
     * @param overrideContentType The content type to override and skip the automatic detection
     * @example
     * ```typescript
     * await stream.setThumbnail("0273f24a-79d1-d0fe-97ca-b0e36bed31es", readFileSync("thumbnail.jpg"))
     * ```
     */
    setThumbnail(videoId: string, thumbnail: Buffer | ReadStream | string, overrideContentType?: string): Promise<BunnyCdnStream.SetThumbnailVideoResponse>;
    /**
     * Upload a video using a URL
     *
     * NOTE: This will not work if the video is not public, and the thumbnail/preview will not be regenerated for existing videos
     * @returns A {@link BunnyCdnStream.FetchVideoResponse} instance
     * @param videoId The video ID
     * @param data The data with video url to fetch and optional headers
     * @example
     * ```typescript
     * await stream.fetchVideo("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { url: "https://example.com/file.mp4" })
     * ```
     */
    fetchVideo(videoId: string, data: {
        url: string;
        headers?: {
            [key: string]: string;
        };
    }): Promise<BunnyCdnStream.FetchVideoResponse>;
    /**
     * Add captions to a video
     * @returns A {@link BunnyCdnStream.AddCaptionsVideoResponse} instance.
     * @param videoId The video ID
     * @param data The data to add captions with where the captions file is a buffer or base64 string
     * @example
     * ```typescript
     * await stream.addCaptions("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { captionsFile: readFile("./subtitles.srt"), label: "English", srclang: "en" })
     * ```
     */
    addCaptions(videoId: string, data: {
        captionsFile: Buffer | string;
        label: string;
        srclang: string;
    }): Promise<BunnyCdnStream.AddCaptionsVideoResponse>;
    /**
     * Delete captions from a video
     * @returns A {@link BunnyCdnStream.DeleteCaptionsVideoResponse} instance.
     * @param videoId The video ID
     * @param srclang The specified srclang used when creating
     * @example
     * ```typescript
     * await stream.deleteCaptions("0273f24a-79d1-d0fe-97ca-b0e36bed31es", "en")
     * ```
     */
    deleteCaptions(videoId: string, srclang: string): Promise<BunnyCdnStream.DeleteCaptionsVideoResponse>;
    /**
     * Create a collection
     * @returns A {@link BunnyCdnStream.CreateCollectionResponse} instance.
     * @param name The collection name
     * @example
     * ```typescript
     * await stream.createCollection("New Collection")
     * ```
     */
    createCollection(name: string): Promise<BunnyCdnStream.CreateCollectionResponse>;
    /**
     * Retrieve info about a collection from BunnyCdn
     * @returns A {@link BunnyCdnStream.BunnyCdnStreamCollection} instance.
     * @param collectionId The collection ID
     * @example
     * ```typescript
     * await stream.getCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    getCollection(collectionId: string): Promise<BunnyCdnStream.BunnyCdnStreamCollection>;
    /**
     * List collections
     * @returns a {@link BunnyCdnStream.ListCollectionsResponse} instances.
     * @param data The options to list collections with
     * @example
     * ```typescript
     * await stream.listCollections({ page: 2, search: "Y collections", itemsPerPage: 100, orderBy: 'date' })
     * ```
     */
    listCollections(data?: {
        page?: number;
        itemsPerPage?: number;
        search?: string;
        orderBy?: string;
    }): Promise<BunnyCdnStream.ListCollectionsResponse>;
    /**
     * List all collections with an optional callback between each page
     * @returns An array of {@link BunnyCdnStream.BunnyCdnStreamCollection} instances.
     * @param data The options to list collections with
     * @param stop The callback that if returns ``true`` stops the iteration
     * @example
     * ```typescript
     * await stream.listAllCollections()
     * ```
     */
    listAllCollections(data?: {
        search?: string;
        orderBy?: string;
        itemsPerPage?: number;
    }, stop?: (collections: BunnyCdnStream.BunnyCdnStreamCollection[], page: number, totalPages: number) => boolean): Promise<BunnyCdnStream.BunnyCdnStreamCollection[]>;
    /**
     * Update info of a collection
     * @returns A {@link BunnyCdnStream.UpdateCollectionResponse} instance.
     * @param collectionId The collection ID
     * @example
     * ```typescript
     * await stream.updateCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es", { name: 'New Collection'})
     * ```
     */
    updateCollection(collectionId: string, data: {
        name: string;
    }): Promise<BunnyCdnStream.UpdateCollectionResponse>;
    /**
     * Delete a collection
     * @returns A {@link BunnyCdnStream.DeleteCollectionResponse} instance.
     * @param collectionId The collection ID
     * @example
     * ```typescript
     * await stream.deleteCollection("0273f24a-79d1-d0fe-97ca-b0e36bed31es")
     * ```
     */
    deleteCollection(collectionId: string): Promise<BunnyCdnStream.DeleteCollectionResponse>;
    /**
     * Delete all collections
     * @returns void
     * @example
     * ```typescript
     * await stream.deleteAllCollections()
     * ```
     */
    deleteAllCollections(): Promise<void>;
    /**
     * Generate a direct upload tus
     *
     * NOTE: metadata.filetype is required for the tus upload to work
     * @returns A {@link BunnyCdnStream.CreateDirectUpload}
     * @param data The data to create the video with
     * @param expirationTime The expiration time of the tus upload
     * @example
     * ```typescript
     * await stream.createDirectUpload({ title: "My Video" })
     * ```
     */
    createDirectUpload(data: {
        title: string;
        collection?: string;
    }, expirationDate?: Date): Promise<BunnyCdnStream.CreateDirectUpload>;
    private generateTUSHash;
    private request;
    private getOptions;
}
declare namespace BunnyCdnStream {
    interface Options {
        videoLibrary: string;
        apiKey: string;
    }
    interface VideoResponse {
        videoLibraryId: number;
        guid: string;
        title: string;
        dateUploaded: string;
        views: number;
        isPublic: boolean;
        length: number;
        status: number;
        framerate: number;
        width: number;
        height: number;
        availableResolutions: string;
        thumbnailCount: number;
        encodeProgress: number;
        storageSize: number;
        captions: {
            srclang: string;
            label: string;
        }[];
        hasMP4Fallback: boolean;
        collectionId: '';
        thumbnailFileName: string;
        averageWatchTime: number;
        totalWatchTime: number;
        category: string;
        chapters: {
            title: string;
            start: number;
            end: number;
        }[];
        moments: {
            label: string;
            timestamp: number;
        }[];
        metaTags: {
            property: string;
            value: string;
        }[];
    }
    interface DeleteVideoResponse {
        success: boolean;
        message?: string;
        statusCode: number;
    }
    interface UploadVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface UpdateVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface AddCaptionsVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface DeleteCaptionsVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface FetchVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface SetThumbnailVideoResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface VideoStatisticsResponse {
        viewsChart: {
            [date: string]: number;
        };
        watchTimeChart: {
            [date: string]: number;
        };
        countryViewCounts: {
            [country: string]: number;
        };
        countryWatchTime: {
            [country: string]: number;
        };
        engagementScore: number;
    }
    interface VideoHeatmapResponse {
    }
    interface ListVideosResponse {
        totalItems: number;
        currentPage: number;
        itemsPerPage: number;
        items: VideoResponse[];
    }
    interface CreateCollectionResponse {
        videoLibraryId: number;
        guid: string;
        name: string;
        videoCount: number;
        totalSize: number;
        previewVideoIds: string;
    }
    interface BunnyCdnStreamCollection {
        videoLibraryId: number;
        guid: string;
        name: string;
        videoCount: number;
        totalSize: number;
        previewVideoIds: string;
    }
    interface ListCollectionsResponse {
        totalItems: number;
        currentPage: number;
        itemsPerPage: number;
        items: BunnyCdnStreamCollection[];
    }
    interface UpdateCollectionResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface DeleteCollectionResponse {
        success: boolean;
        message: string;
        statusCode: number;
    }
    interface CreateDirectUpload {
        video: BunnyCdnStreamVideo;
        endpoint: string;
        headers: {
            AuthorizationSignature: string;
            AuthorizationExpire: number;
            VideoId: string;
            LibraryId: string;
        };
        metadata: {
            filetype: string;
            title: string;
            collection: string | undefined;
        };
    }
    interface BunnyAxiosRequestConfig extends AxiosRequestConfig {
        headers: AxiosRequestHeaders;
    }
}

export { BunnyCdnStream, BunnyCdnStreamVideo };
